==> arc-cli-6.3.0b3.tar.gz: arc-cli-6.3.0b3/arc/params.py <==
"""Public API for hanlding parameter modification"""
from __future__ import annotations
import typing as t
from arc._command import param
from arc import constants


class ParamInfo:
    def __init__(
        self,
        param_cls: type[param.Param] = None,
        arg_alias: str = None,
        short: str = None,
        default: t.Any = constants.MISSING,
        description: str = None,
        callback: t.Callable = None,
        action: param.ParamAction = None,
        prompt: str = None,
        envvar: str = None,
    ):
        self.param_cls = param_cls
        self.arg_alias = arg_alias
        self.short = short
        self.default = default
        self.description = description
        self.callback = callback
        self.action = action
        self.prompt = prompt
        self.envvar = envvar

    def dict(self):
        """Used to pass to `Param()` as **kwargs"""
        return {
            "arg_alias": self.arg_alias,
            "short": self.short,
            "default": self.default,
            "description": self.description,
            "callback": self.callback,
            "action": self.action,
            "prompt": self.prompt,
            "envvar": self.envvar,
        }


def Param(
    *,
    name: str = None,
    short: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
) -> t.Any:
    """A CLI Paramater. Automatically decides whether it is
    a `positional`, `option` or `flag`

    # Example
    ```py
    @cli.command()
    def test(val: int = Param(), *, val2: int = Param(), flag: bool = Param()):
        print(val, val2, flag)
    ```
    Each Param type will be determined as follows:
    - `val`:  Positional argument because it precedes the bare `*`.
    - `val2`: Option argument because  it proceeds the bare `*`.
              Python considers this a "keyword only" argument, and so does arc
    - `flag`: Flag argument because it has `bool` type

    ```
    $ python example.py test --val2 3 --flag -- 2
    2 3 True
    ```
    """
    return ParamInfo(
        None,
        arg_alias=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
    )


def Argument(
    *,
    name: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
) -> t.Any:
    """A CLI Paramater. Input will be passed in positionally.

    # Example
    ```py
    @cli.command()
    def test(val: int = Argument()):
        print(val)
    ```

    ```
    $ python example.py test 2
    2
    ```
    """
    return ParamInfo(
        param_cls=param.Argument,
        arg_alias=name,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
    )


def Option(
    *,
    name: str = None,
    short: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
) -> t.Any:
    """A (generally optional) keyword parameter.

      # Example
    ```py
    @cli.command()
    def test(val: int = Option()):
        print(val)
    ```

    ```
    $ python example.py test --val 2
    2
    ```
    """
    return ParamInfo(
        param_cls=param.Option,
        arg_alias=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
    )


def Flag(
    *,
    name: str = None,
    short: str = None,
    default: bool = False,
    description: str = None,
    callback: t.Callable = None,
) -> t.Any:
    """An option that represents a boolean value.

    # Example
    ```py
    @cli.command()
    def test(val: bool = Flag()):
        print(val)
    ```

    ```
    $ python example.py test
    False
    $ python example.py test --flag
    True
    ```
    """
    return ParamInfo(
        param_cls=param.Flag,
        arg_alias=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
    )


def Count(
    *,
    name: str = None,
    short: str = None,
    default: int = 0,
    description: str = None,
    callback: t.Callable = None,
) -> t.Any:
    """A Flag that counts it's number of apperances on the command line

    # Example
    ```py
    @cli.command()
    def test(val: int = Count(short="v")):
        print(val)
    ```

    ```
    $ python example.py test
    0
    $ python example.py test --val
    1
    $ python example.py test -vvvv
    4
    ```
    """
    return ParamInfo(
        param_cls=param.Flag,
        arg_alias=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        action=param.ParamAction.COUNT,
    )


def SpecialParam(
    name: str = None,
    short: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
) -> t.Any:
    """Params marked as "Special" are not exposed to the command line
    interface and cannot recieve user input. As such, they're values
    are expected to come from elsewhere. This allows commands to recieve
    their values like regular params, but still have them act in a particular
    way.

    It is primarly used for some builtin-types like `State` and `Context`.
    """
    return ParamInfo(
        param_cls=param.SpecialParam,
        arg_alias=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
    )


T = t.TypeVar("T")


def __cls_deco_factory(param_cls: type[param.Param]):
    def decorator(
        name: str = None,
        short: str = None,
        default: t.Any = constants.MISSING,
        description: str = None,
        overwrite: bool = False,
    ):
        def inner(cls: T) -> T:
            setattr(
                cls,
                "__param_info__",
                {
                    "param_cls": param_cls,
                    "arg_alias": name,
                    "short": short,
                    "default": default,
                    "description": description,
                    "overwrite": overwrite,
                },
            )
            return cls

        return inner

    return decorator


argument = __cls_deco_factory(param.Argument)
option = __cls_deco_factory(param.Option)
flag = __cls_deco_factory(param.Flag)
special = __cls_deco_factory(param.SpecialParam)

==> arc-cli-7.0.0.tar.gz: arc-cli-7.0.0/arc/params.py <==
import typing as t

from arc import constants
from arc._command.classful import lazy_class_signature
from arc._command.param import param
import arc.typing as at


class ParamInfo:
    def __init__(
        self,
        param_cls: type[param.Param],
        param_name: str = None,
        short: str = None,
        default: t.Any = constants.MISSING,
        description: str = None,
        callback: t.Callable = None,
        action: param.Action = None,
        prompt: str = None,
        envvar: str = None,
        complete: at.CompletionFunc = None,
        getter_func: at.GetterFunc = None,
    ):
        self.param_cls = param_cls
        self.param_name = param_name
        self.short_name = short
        self.default = default
        self.description = description
        self.callback = callback
        self.action = action
        self.prompt = prompt
        self.envvar = envvar
        self.complete = complete
        self.getter_func = getter_func

    def dict(self):
        """Used to pass to `Param()` as **kwargs"""
        return {
            "param_name": self.param_name,
            "short_name": self.short_name,
            "default": self.default,
            "description": self.description,
            "callback": self.callback,
            "prompt": self.prompt,
            "envvar": self.envvar,
            "action": self.action,
            "comp_func": self.complete,
            "getter_func": self.getter_func,
        }


def Argument(
    *,
    name: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
    get: at.GetterFunc = None,
    complete: at.CompletionFunc = None,
) -> t.Any:
    """A CLI Paramater. Input will be passed in positionally.

    # Example
    ```py
    @cli.command()
    def test(val: int = Argument()):
        arc.print(val)
    ```

    ```
    $ python example.py test 2
    2
    ```
    """
    return ParamInfo(
        param_cls=param.ArgumentParam,
        param_name=name,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
        getter_func=get,
        complete=complete,
    )


def Option(
    *,
    name: str = None,
    short: str = None,
    default: t.Any = constants.MISSING,
    description: str = None,
    callback: t.Callable = None,
    prompt: str = None,
    envvar: str = None,
    get: at.GetterFunc = None,
    complete: at.CompletionFunc = None,
) -> t.Any:
    """A (generally optional) keyword parameter.

    # Example
    ```py
    @cli.command()
    def test(val: int = Option()):
        arc.print(val)
    ```

    ```
    $ python example.py test --val 2
    2
    ```
    """

    return ParamInfo(
        param_cls=param.OptionParam,
        param_name=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        prompt=prompt,
        envvar=envvar,
        getter_func=get,
        complete=complete,
    )


def Flag(
    *,
    name: str = None,
    short: str = None,
    default: bool = False,
    description: str = None,
    callback: t.Callable = None,
) -> t.Any:
    """An option that represents a boolean value.

    # Example
    ```py
    @cli.command()
    def test(val: bool = Flag()):
        arc.print(val)
    ```

    ```
    $ python example.py test
    False
    $ python example.py test --flag
    True
    ```
    """
    return ParamInfo(
        param_cls=param.FlagParam,
        param_name=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
    )


def Count(
    *,
    name: str = None,
    short: str = None,
    default: int = 0,
    description: str = None,
    callback: t.Callable = None,
) -> t.Any:
    """A Flag that counts it's number of apperances on the command line

    # Example
    ```py
    @cli.command()
    def test(val: int = Count(short="v")):
        arc.print(val)
    ```

    ```
    $ python example.py test
    0
    $ python example.py test --val
    1
    $ python example.py test -vvvv
    4
    ```
    """
    return ParamInfo(
        param_cls=param.FlagParam,
        param_name=name,
        short=short,
        default=default,
        description=description,
        callback=callback,
        action=param.Action.COUNT,
    )


def Depends(callback: t.Callable) -> t.Any:
    return ParamInfo(param_cls=param.InjectedParam, callback=callback)


G = t.TypeVar("G", bound=type)


def group(cls: G) -> G:
    setattr(cls, "__arc_group__", True)
    lazy_class_signature(cls)
    return cls
